@@ -231,7 +231,6 @@ XLogDumpXLogRead(const char *directory, TimeLineID timeline_id,
 		if (sendFile < 0 || !XLByteInSeg(recptr, sendSegNo))
 		{
 			char		fname[MAXFNAMELEN];
-			int			tries;
 
 			/* Switch to another logfile segment */
 			if (sendFile >= 0)
@@ -241,30 +240,7 @@ XLogDumpXLogRead(const char *directory, TimeLineID timeline_id,
 
 			XLogFileName(fname, timeline_id, sendSegNo);
 
-			/*
-			 * In follow mode there is a short period of time after the
-			 * server has written the end of the previous file before the
-			 * new file is available. So we loop for 5 seconds looking
-			 * for the file to appear before giving up.
-			 */
-			for (tries = 0; tries < 10; tries++)
-			{
-				sendFile = fuzzy_open_file(directory, fname);
-				if (sendFile >= 0)
-					break;
-				if (errno == ENOENT)
-				{
-					int			save_errno = errno;
-
-					/* File not there yet, try again */
-					pg_usleep(500 * 1000);
-
-					errno = save_errno;
-					continue;
-				}
-				/* Any other error, fall through and fail */
-				break;
-			}
+			sendFile = fuzzy_open_file(directory, fname);
 
 			if (sendFile < 0)
 				fatal_error("could not find file \"%s\": %s",
@@ -583,7 +559,7 @@ main(int argc, char **argv)
 		if (!verify_directory(private.inpath))
 		{
 			fprintf(stderr,
-					"%s: path \"%s\" cannot be opened: %s\n",
+					"%s: path \"%s\" cannot be opened: %s",
 					progname, private.inpath, strerror(errno));
 			goto bad_argument;
 		}
@@ -720,7 +696,7 @@ main(int argc, char **argv)
 	}
 
 	if (errormsg)
-		fatal_error("error in WAL record at %X/%X: %s",
+		fatal_error("error in WAL record at %X/%X: %s\n",
 					(uint32) (xlogreader_state->ReadRecPtr >> 32),
 					(uint32) xlogreader_state->ReadRecPtr,
 					errormsg);
