@@ -4,7 +4,7 @@
  *	  This file contains routines to support creation of toast tables
  *
  *
- * Portions Copyright (c) 1996-2012, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
@@ -20,6 +20,7 @@
 #include "catalog/heap.h"
 #include "catalog/index.h"
 #include "catalog/namespace.h"
+#include "catalog/pg_authid.h"
 #include "catalog/pg_namespace.h"
 #include "catalog/pg_opclass.h"
 #include "catalog/pg_type.h"
@@ -39,6 +40,9 @@ static bool create_toast_table(Relation rel, Oid toastOid, Oid toastIndexOid,
 				   Datum reloptions);
 static bool needs_toast_table(Relation rel);
 
+/* pick a OID that will never be used for TOAST tables */
+#define OPTIONALLY_CREATE_TOAST_OID	BOOTSTRAP_SUPERUSERID
+
 
 /*
  * AlterTableCreateToastTable
@@ -84,12 +88,11 @@ BootstrapToastTable(char *relName, Oid toastOid, Oid toastIndexOid)
 
 	rel = heap_openrv(makeRangeVar(NULL, relName, -1), AccessExclusiveLock);
 
-	/* Note: during bootstrap may see uncataloged relation */
 	if (rel->rd_rel->relkind != RELKIND_RELATION &&
-		rel->rd_rel->relkind != RELKIND_UNCATALOGED)
+		rel->rd_rel->relkind != RELKIND_MATVIEW)
 		ereport(ERROR,
 				(errcode(ERRCODE_WRONG_OBJECT_TYPE),
-				 errmsg("\"%s\" is not a table",
+				 errmsg("\"%s\" is not a table or materialized view",
 						relName)));
 
 	/* create_toast_table does all the work */
@@ -151,16 +154,51 @@ create_toast_table(Relation rel, Oid toastOid, Oid toastIndexOid, Datum reloptio
 	if (rel->rd_rel->reltoastrelid != InvalidOid)
 		return false;
 
-	/*
-	 * Check to see whether the table actually needs a TOAST table.
-	 *
-	 * If an update-in-place toast relfilenode is specified, force toast file
-	 * creation even if it seems not to need one.
-	 */
-	if (!needs_toast_table(rel) &&
-		(!IsBinaryUpgrade ||
-		 !OidIsValid(binary_upgrade_next_toast_pg_class_oid)))
-		return false;
+	if (!IsBinaryUpgrade)
+	{
+		if (!needs_toast_table(rel))
+			return false;
+	}
+	else
+	{
+		/*
+		 * Check to see whether the table needs a TOAST table.
+		 *
+		 * If an update-in-place TOAST relfilenode is specified, force TOAST file
+		 * creation even if it seems not to need one.  This handles the case
+		 * where the old cluster needed a TOAST table but the new cluster
+		 * would not normally create one.
+		 */
+
+		/*
+		 * If a TOAST oid is not specified, skip TOAST creation as we will do
+		 * it later so we don't create a TOAST table whose OID later conflicts
+		 * with a user-supplied OID.  This handles cases where the old cluster
+		 * didn't need a TOAST table, but the new cluster does.
+		 */
+		if (!OidIsValid(binary_upgrade_next_toast_pg_class_oid))
+			return false;
+
+		/*
+		 * If a special TOAST value has been passed in, it means we are in
+		 * cleanup mode --- we are creating needed TOAST tables after all user
+		 * tables with specified OIDs have been created.  We let the system
+		 * assign a TOAST oid for us.  The tables are empty so the missing
+		 * TOAST tables were not a problem.
+		 */
+		if (binary_upgrade_next_toast_pg_class_oid == OPTIONALLY_CREATE_TOAST_OID)
+		{
+			/* clear as it is not to be used; it is just a flag */
+			binary_upgrade_next_toast_pg_class_oid = InvalidOid;
+
+			if (!needs_toast_table(rel))
+				return false;
+		}
+
+		/* both should be set, or not set */
+		Assert(OidIsValid(binary_upgrade_next_toast_pg_class_oid) ==
+			   OidIsValid(binary_upgrade_next_toast_pg_type_oid));
+	}
 
 	/*
 	 * Create the toast table and its index
@@ -228,6 +266,7 @@ create_toast_table(Relation rel, Oid toastOid, Oid toastIndexOid, Datum reloptio
 										   ONCOMMIT_NOOP,
 										   reloptions,
 										   false,
+										   true,
 										   true);
 	Assert(toast_relid != InvalidOid);
 
@@ -281,7 +320,7 @@ create_toast_table(Relation rel, Oid toastOid, Oid toastIndexOid, Datum reloptio
 				 rel->rd_rel->reltablespace,
 				 collationObjectId, classObjectId, coloptions, (Datum) 0,
 				 true, false, false, false,
-				 true, false, false);
+				 true, false, false, true);
 
 	heap_close(toast_rel, NoLock);
 
