@@ -3,7 +3,7 @@
  * dict_ispell.c
  *		Ispell dictionary interface
  *
- * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
  *
  *
  * IDENTIFICATION
@@ -16,9 +16,7 @@
 #include "commands/defrem.h"
 #include "tsearch/dicts/spell.h"
 #include "tsearch/ts_locale.h"
-#include "tsearch/ts_public.h"
 #include "tsearch/ts_utils.h"
-#include "utils/builtins.h"
 
 
 typedef struct
@@ -128,20 +126,19 @@ dispell_lexize(PG_FUNCTION_ARGS)
 	if (res == NULL)
 		PG_RETURN_POINTER(NULL);
 
-	ptr = cptr = res;
-	while (ptr->lexeme)
+	cptr = res;
+	for (ptr = cptr; ptr->lexeme; ptr++)
 	{
 		if (searchstoplist(&(d->stoplist), ptr->lexeme))
 		{
 			pfree(ptr->lexeme);
 			ptr->lexeme = NULL;
-			ptr++;
 		}
 		else
 		{
-			memcpy(cptr, ptr, sizeof(TSLexeme));
+			if (cptr != ptr)
+				memcpy(cptr, ptr, sizeof(TSLexeme));
 			cptr++;
-			ptr++;
 		}
 	}
 	cptr->lexeme = NULL;
