@@ -169,19 +169,6 @@ findchar(char *str, int c)
 	return NULL;
 }
 
-static char *
-findchar2(char *str, int c1, int c2)
-{
-	while (*str)
-	{
-		if (t_iseq(str, c1) || t_iseq(str, c2))
-			return str;
-		str += pg_mblen(str);
-	}
-
-	return NULL;
-}
-
 
 /* backward string compare for suffix tree operations */
 static int
@@ -470,149 +457,13 @@ NIAddAffix(IspellDict *Conf, int flag, char flagflags, const char *mask, const c
 	Conf->naffixes++;
 }
 
-
-/* Parsing states for parse_affentry() and friends */
 #define PAE_WAIT_MASK	0
-#define PAE_INMASK		1
+#define PAE_INMASK	1
 #define PAE_WAIT_FIND	2
-#define PAE_INFIND		3
+#define PAE_INFIND	3
 #define PAE_WAIT_REPL	4
-#define PAE_INREPL		5
-#define PAE_WAIT_TYPE	6
-#define PAE_WAIT_FLAG	7
+#define PAE_INREPL	5
 
-/*
- * Parse next space-separated field of an .affix file line.
- *
- * *str is the input pointer (will be advanced past field)
- * next is where to copy the field value to, with null termination
- *
- * The buffer at "next" must be of size BUFSIZ; we truncate the input to fit.
- *
- * Returns TRUE if we found a field, FALSE if not.
- */
-static bool
-get_nextfield(char **str, char *next)
-{
-	int			state = PAE_WAIT_MASK;
-	int			avail = BUFSIZ;
-
-	while (**str)
-	{
-		if (state == PAE_WAIT_MASK)
-		{
-			if (t_iseq(*str, '#'))
-				return false;
-			else if (!t_isspace(*str))
-			{
-				int			clen = pg_mblen(*str);
-
-				if (clen < avail)
-				{
-					COPYCHAR(next, *str);
-					next += clen;
-					avail -= clen;
-				}
-				state = PAE_INMASK;
-			}
-		}
-		else	/* state == PAE_INMASK */
-		{
-			if (t_isspace(*str))
-			{
-				*next = '\0';
-				return true;
-			}
-			else
-			{
-				int			clen = pg_mblen(*str);
-
-				if (clen < avail)
-				{
-					COPYCHAR(next, *str);
-					next += clen;
-					avail -= clen;
-				}
-			}
-		}
-		*str += pg_mblen(*str);
-	}
-
-	*next = '\0';
-
-	return (state == PAE_INMASK);		/* OK if we got a nonempty field */
-}
-
-/*
- * Parses entry of an .affix file of MySpell or Hunspell format.
- *
- * An .affix file entry has the following format:
- * - header
- *	 <type>  <flag>  <cross_flag>  <flag_count>
- * - fields after header:
- *	 <type>  <flag>  <find>  <replace>	<mask>
- *
- * str is the input line
- * field values are returned to type etc, which must be buffers of size BUFSIZ.
- *
- * Returns number of fields found; any omitted fields are set to empty strings.
- */
-static int
-parse_ooaffentry(char *str, char *type, char *flag, char *find,
-				 char *repl, char *mask)
-{
-	int			state = PAE_WAIT_TYPE;
-	int			fields_read = 0;
-	bool		valid = false;
-
-	*type = *flag = *find = *repl = *mask = '\0';
-
-	while (*str)
-	{
-		switch (state)
-		{
-			case PAE_WAIT_TYPE:
-				valid = get_nextfield(&str, type);
-				state = PAE_WAIT_FLAG;
-				break;
-			case PAE_WAIT_FLAG:
-				valid = get_nextfield(&str, flag);
-				state = PAE_WAIT_FIND;
-				break;
-			case PAE_WAIT_FIND:
-				valid = get_nextfield(&str, find);
-				state = PAE_WAIT_REPL;
-				break;
-			case PAE_WAIT_REPL:
-				valid = get_nextfield(&str, repl);
-				state = PAE_WAIT_MASK;
-				break;
-			case PAE_WAIT_MASK:
-				valid = get_nextfield(&str, mask);
-				state = -1;		/* force loop exit */
-				break;
-			default:
-				elog(ERROR, "unrecognized state in parse_ooaffentry: %d",
-					 state);
-				break;
-		}
-		if (valid)
-			fields_read++;
-		else
-			break;				/* early EOL */
-		if (state < 0)
-			break;				/* got all fields */
-	}
-
-	return fields_read;
-}
-
-/*
- * Parses entry of an .affix file of Ispell format
- *
- * An .affix file entry has the following format:
- * <mask>  >  [-<find>,]<replace>
- */
 static bool
 parse_affentry(char *str, char *mask, char *find, char *repl)
 {
@@ -767,6 +618,8 @@ NIImportOOAffixes(IspellDict *Conf, const char *filename)
 	int			flag = 0;
 	char		flagflags = 0;
 	tsearch_readline_state trst;
+	int			scanread = 0;
+	char		scanbuf[BUFSIZ];
 	char	   *recoded;
 
 	/* read file to find any flag */
@@ -829,6 +682,8 @@ NIImportOOAffixes(IspellDict *Conf, const char *filename)
 	}
 	tsearch_readline_end(&trst);
 
+	sprintf(scanbuf, "%%6s %%%ds %%%ds %%%ds %%%ds", BUFSIZ / 5, BUFSIZ / 5, BUFSIZ / 5, BUFSIZ / 5);
+
 	if (!tsearch_readline_begin(&trst, filename))
 		ereport(ERROR,
 				(errcode(ERRCODE_CONFIG_FILE_ERROR),
@@ -837,21 +692,18 @@ NIImportOOAffixes(IspellDict *Conf, const char *filename)
 
 	while ((recoded = tsearch_readline(&trst)) != NULL)
 	{
-		int			fields_read;
-
 		if (*recoded == '\0' || t_isspace(recoded) || t_iseq(recoded, '#'))
 			goto nextline;
 
-		fields_read = parse_ooaffentry(recoded, type, sflag, find, repl, mask);
+		scanread = sscanf(recoded, scanbuf, type, sflag, find, repl, mask);
 
 		if (ptype)
 			pfree(ptype);
 		ptype = lowerstr_ctx(Conf, type);
-		if (fields_read < 4 ||
-			(STRNCMP(ptype, "sfx") != 0 && STRNCMP(ptype, "pfx") != 0))
+		if (scanread < 4 || (STRNCMP(ptype, "sfx") && STRNCMP(ptype, "pfx")))
 			goto nextline;
 
-		if (fields_read == 4)
+		if (scanread == 4)
 		{
 			if (strlen(sflag) != 1)
 				goto nextline;
@@ -869,20 +721,18 @@ NIImportOOAffixes(IspellDict *Conf, const char *filename)
 
 			if (strlen(sflag) != 1 || flag != *sflag || flag == 0)
 				goto nextline;
-			/* Get flags after '/' (flags are case sensitive) */
-			if ((ptr = strchr(repl, '/')) != NULL)
+			prepl = lowerstr_ctx(Conf, repl);
+			/* affix flag */
+			if ((ptr = strchr(prepl, '/')) != NULL)
 			{
-				ptr++;
+				*ptr = '\0';
+				ptr = repl + (ptr - prepl) + 1;
 				while (*ptr)
 				{
 					aflg |= Conf->flagval[*(unsigned char *) ptr];
 					ptr++;
 				}
 			}
-			/* Get lowercased version of string before '/' */
-			prepl = lowerstr_ctx(Conf, repl);
-			if ((ptr = strchr(prepl, '/')) != NULL)
-				*ptr = '\0';
 			pfind = lowerstr_ctx(Conf, find);
 			pmask = lowerstr_ctx(Conf, mask);
 			if (t_iseq(find, '0'))
@@ -950,10 +800,11 @@ NIImportAffixes(IspellDict *Conf, const char *filename)
 
 		if (STRNCMP(pstr, "compoundwords") == 0)
 		{
-			/* Find case-insensitive L flag in non-lowercased string */
-			s = findchar2(recoded, 'l', 'L');
+			s = findchar(pstr, 'l');
 			if (s)
 			{
+				s = recoded + (s - pstr);		/* we need non-lowercased
+												 * string */
 				while (*s && !t_isspace(s))
 					s += pg_mblen(s);
 				while (*s && t_isspace(s))
@@ -1370,10 +1221,8 @@ NISortAffixes(IspellDict *Conf)
 		if ((Affix->flagflags & FF_COMPOUNDFLAG) && Affix->replen > 0 &&
 			isAffixInUse(Conf, (char) Affix->flag))
 		{
-			bool		issuffix = (Affix->type == FF_SUFFIX);
-
 			if (ptr == Conf->CompoundAffix ||
-				issuffix != (ptr - 1)->issuffix ||
+				ptr->issuffix != (ptr - 1)->issuffix ||
 				strbncmp((const unsigned char *) (ptr - 1)->affix,
 						 (const unsigned char *) Affix->repl,
 						 (ptr - 1)->len))
@@ -1381,7 +1230,7 @@ NISortAffixes(IspellDict *Conf)
 				/* leave only unique and minimals suffixes */
 				ptr->affix = Affix->repl;
 				ptr->len = Affix->replen;
-				ptr->issuffix = issuffix;
+				ptr->issuffix = (Affix->type == FF_SUFFIX) ? true : false;
 				ptr++;
 			}
 		}
